--[[

	TODO: destroy button when finished crafting, so that you can right click again to complete quest
	TODO: if button was closed. maybe right click without closing quest dialog should bring button back
	TODO: respect quest progress 1/2 (one item crafted already)?
	TODO: CloseTradeSkill() is called before second crafting has begun
]]
local GH = GarrisonHelper
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};

GH.jewelcraftingquests = {
	--[[
		quest name, crafted item id, crafted item spell id, crafted item amount, required resources and amount
		unfortunately we have to note the required reagents, because opening a trade skill window is one click too much
	]]
	[37321] = {"Jewelcrafting Special Order: A Yellow Brighter Than Gold", 115803,170719,2,{[109118]=10,[109125]=2}},
	[37323] = {"Jewelcrafting Special Order: Blue the Shade of Sky and Sea", 115804,170720,2,{[109118]=10,[109124]=2}},
	[37319] = {"Jewelcrafting Special Order: Wedding Bands", 115993,170710,2,{[109119]=120}},
	[37320] = {"Jewelcrafting Special Order: A Fine Choker", 115992,170709,1,{[109118]=60}},
	[37324] = {"Out of Stock: Blackrock Ore", nil, nil, nil,{[109118]=20}},
	[37325] = {"Out of Stock: True Iron", nil, nil, nil,{[109119]=20}},
};
local jewelcraftingID = 132; -- jewelcrafting building

function GH:isJewelcraftingEnabled()
	if type(GH.db.profile.jewelcraftingsupport)==nil then GH.db.profile.jewelcraftingsupport=true end
	return GH.db.profile.jewelcraftingsupport
end

function GH:isJewelcraftingQuest(qID)
	return GH.jewelcraftingquests[qID] and GH:isJewelcraftingEnabled() and GH:isSelectedJewelcraftingDaily(qID) 
end

function events.QUEST_DETAIL()
	if GH:isJewelcraftingQuest(GetQuestID()) then
		if GH:QuestCanBeCompleted(GetQuestID()) then 
			AcceptQuest();
		end
	end
end

function events.QUEST_ACCEPTED(qIndex, qID)
	if GH:isJewelcraftingQuest(GetQuestID()) then
		GH_craft()
	end
end

function events.QUEST_WATCH_UPDATE(qIndex)
	-- "At this point the game client's quest data is not yet updated, but will be after a subsequent QUEST_LOG_UPDATE event."
	GH.questwatchupdateindex = qIndex
end
function events.QUEST_LOG_UPDATE()
	if GH.questwatchupdateindex then
		local _, _, _, _, _, _, _, questID = GetQuestLogTitle(GH.questwatchupdateindex);
		GH.questwatchupdateindex = nil
		if GH:isJewelcraftingQuest(questID) then
			GH_craft()
		end
	end
end

function events.QUEST_COMPLETE()
	if GH:isJewelcraftingQuest(GetQuestID()) then
		GetQuestReward();
	end
end

function events.QUEST_PROGRESS()
	if GH:isJewelcraftingQuest(GetQuestID()) then
		if IsQuestCompletable() then
			CompleteQuest()
		else
			if GH:QuestCanBeCompleted(GetQuestID()) then 
				GH_craft()
			end
		end
	end
end
--fallback for not knowing jewelcrafting and talking to an NPC
function events.TRADE_SKILL_SHOW()
	if not GH:hasProfessionJewelcrafting() then
		GH_craft()
	end
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event,...) events[event](...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end

function GH:getProfessionOfTradeskillWindow()
	local link = GetTradeSkillListLink()
	if link then
		local _,_,professionSpellID,professionID,professionName =  string.find(link, "(%d+):(%d+)|?h?%[(.+)%]")
		return professionName,tonumber(professionID),tonumber(professionSpellID)
	end
end

function GH_craft()
	local questIndex, is, should, obj, sLink, questID
	--search active jc quest
	for questLogID=1,GetNumQuestLogEntries() do
		_, _, _, _, _, _, _, questID = GetQuestLogTitle(questLogID);
		if GH.jewelcraftingquests[questID] then questIndex=questLogID break end
	end
	if questIndex and GH.jewelcraftingquests[questID][2] then -- if quest was accepted and has something to craft
		local isComplete = true
		for i=1,GetNumQuestLeaderBoards(questIndex) do
			local _, _, finished = GetQuestObjectiveInfo(questID, i)
			isComplete = isComplete and finished
		end
		if isComplete then
			--quest objective reached, remove action button
			GH:destroy("jc")
			CloseQuest()
		else
			local sID = GH.jewelcraftingquests[questID][2]
			local spellID = GH.jewelcraftingquests[questID][3]
			local objective, sLink = GetItemInfo(sID)
			local num = GH.jewelcraftingquests[questID][4]
			
			local tooltiptext = ""
			if sLink then
				tooltiptext = tooltiptext .. "to create " .. num .. " x " .. sLink .. "\nconsumes: "
			end
			tooltiptext = tooltiptext .. GH:JewelcraftingQuestResources(questID)
			if objective then
				--[[
					Workaround: This crafting needs no open tradeskill window. It picks up a crafting spell,
					places the spell somewhere on an empty action bar slot, uses the action and then clears
					the action bar slot
				]]
				--searches empty action bar slot
				--/run local emptySlot; for l=1, 120 do if not GetActionTexture(l) then emptySlot=l end end ClearCursor() PickupSpell(26925) PlaceAction(emptySlot) UseAction(emptySlot) PickupAction(emptySlot) ClearCursor()
				local emptySlot;
				for l=1, 120 do
					if not GetActionTexture(l) then emptySlot=l end -- get the last empty slot
				end
				local m
				if emptySlot then
					m = "/run ClearCursor() PickupSpell(" .. spellID .. ") PlaceAction(" .. emptySlot .. ") UseAction(" .. emptySlot .. ") PickupAction(" .. emptySlot .. ") ClearCursor() GH_destroy('jc')"
				else
					m = "/run CloseTradeSkill()\n/cast Jewelcrafting\n/run for i=1,GetNumTradeSkills() do if (GetTradeSkillInfo(i)=='" .. objective .. "') then DoTradeSkill(i) GH_destroy('jc') break end end CloseTradeSkill()"
				end
				
				if IsNPCCrafting() then
					num = 1 -- I think it is risky to craft two items at once. what if you already crafted 1/2
					GH_CraftingNPCCanDoTradeSkill(objective,num,sID,tooltiptext)
				elseif GH:hasProfessionJewelcrafting() then
					GH:setActionButton(sID, m, tooltiptext,nil,nil,nil,"jc")
				else
					GH:print("Click the NPC to craft the quest objective!")
				end
			end
		end
	end
end

--[[ 
	Called until the tradeskills of NPC are loaded.
	Then, it is looked, if he can help with creating the quest item.
]]
function GH_CraftingNPCCanDoTradeSkill(item,num,sID,tooltiptext)
	-- count errors
	GH.nojctradeskillcount = GH.nojctradeskillcount or 0
	if GetNumTradeSkills()==0 then
		--try again
		GH.nojctradeskillcount = GH.nojctradeskillcount+1
		if GH.nojctradeskillcount <= 10 then
			C_Timer.After(0.5, function() GH_CraftingNPCCanDoTradeSkill(item,num,sID,tooltiptext) end)
		else
			GH.nojctradeskillcount = 0
		end
	else
		for i=1,GetNumTradeSkills() do
			if GetTradeSkillInfo(i)==item then
				GH:setActionButton(sID, "/run DoTradeSkill(" .. i .. "," .. num .. ")", tooltiptext,nil,nil,nil,"jc")
				break
			end
		end
	end
end

function GH:JewelcraftingQuestResources(questID)
	local res = ""
	for item, numItems in pairs(GH.jewelcraftingquests[questID][5]) do
		if #res>0 then res = res .. ", " end
		local v = select(2,GetItemInfo(item))
		res = res .. numItems .. " x " .. tostring(v)
	end
	return res
end

function GH:JewelcraftingQuestTooltip(questID)
	local res = ""
	--[[
	local o
	for i=1,1 do
		o = GetQuestObjectiveInfo(questID,i)
		if o then
			if #res>0 then res = res .. "\n" end
			res = res .. tostring(o)
		else
			break
		end
	end
	res = "quest objective:\n" .. res
	]]
	for item, numItems in pairs(GH.jewelcraftingquests[questID][5]) do
		if #res>0 then res = res .. ", " end
		local itemTexture = select(10,GetItemInfo(item))
		res = res .. numItems .. " x " .. "\124T" .. itemTexture  ..":24\124t"
	end
	return res
end

--[[
	check if required materials for completing or crafting is available
]]
function GH:QuestCanBeCompleted(questID)
	local res = true
	if GH:isJewelcraftingQuest(questID) then
		for item, numItems in pairs(GH.jewelcraftingquests[questID][5]) do
			-- check if required items are in inventory (and bank)
			if numItems>GetItemCount(item, true) then
				print("|r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF \124cffffff00 Not enough reagents ("..GetItemInfo(item).." "..GetItemCount(item, true).."/"..numItems..")");
				res = false
			end
		end
	end
	return res
end	

function GH:isSelectedJewelcraftingDaily(qID)
	local res = true
	for i=1,10 do
		if GH.db.profile["jewelcraftingquest" .. i .. "id"]==qID then
			-- if nil or true -> yes: no otherwise
			if type(GH.db.profile["jewelcraftingquest" .. i])=="boolean" and not GH.db.profile["jewelcraftingquest" .. i] then
				res = false
				break
			end
		end
	end
	return res
end

function GH:hasProfessionJewelcrafting()

	local res = false
	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions("player");
	local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(prof1)
	if skillLine == 755 then
		res = true
	else
		name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(prof2)
		if skillLine == 755 then
			res = true
		end
	end
	return res
end

function GH_jewelcraftingWelcome()
	if GH:isJewelcraftingEnabled() and GH:hasBuilding(jewelcraftingID,2,true) then
		local done = false
		for questID, _ in pairs(GH.jewelcraftingquests) do
			done = done or IsQuestFlaggedCompleted(questID)
		end
		
		if not done then
			GH:print("Jewelcrafting daily not completed today");
		end
	end
end

--[[
	Needed, because if cache is empty, this information has to be retrieved from server.
]]
function GH:PreLoadJewelcraftingItemInfos()
	for questID, info in pairs(GH.jewelcraftingquests) do
		for item, _ in pairs(GH.jewelcraftingquests[questID][5]) do
			GetItemInfo(item)
		end
		if GH.jewelcraftingquests[questID][2] then
			GetItemInfo(GH.jewelcraftingquests[questID][2])
		end
	end
end
GH:PreLoadJewelcraftingItemInfos() -- TODO move this to an event like addon loaded