local GH = GarrisonHelper
-- Frame and Events
local GarrisonHelper_Events = CreateFrame("Frame");
local events = {};
local ids = {
	["Partial Receipt"] = 119126,
	["Milling"] = 51005,
	["Milling Icon"] = 114942,
	["Cerulean Pigment"] = 114931,
}
local inscriptionID = 130;

local millingPriority = {
	"Fireweed","Frostweed","Nagrand Arrowbloom","Starflower","Talador Orchid","Gorgrond Flytrap"
}


function events.TRADE_SKILL_SHOW()
	--print("TRADE_SKILL_SHOW " .. tostring(GH.inprogress))
	GH:doMillingDaily()
end
--[[
function events.UPDATE_TRADESKILL_RECAST()
	print("UPDATE_TRADESKILL_RECAST " .. tostring(GH.inprogress))
	-- may happen when trade skill was not loaded when showing
	GH:doMillingDaily()
end
]]
--[[
function events.UNIT_SPELLCAST_SUCCEEDED(self, event, target, spell)
	print("UNIT_SPELLCAST_SUCCEEDED " .. spell)
	-- if milling was performed
	if spell and spell == "Milling" then
		waitFor = true
	end
end
]]
function events.BAG_UPDATE()
	--print("BAG_UPDATE " .. tostring(GH.inprogress))
	-- if milling was performed, then pigments have been looted
	GH:doMillingDaily()
end

function events.BAG_UPDATE_DALAYED()
	--print("BAG_UPDATE_DALAYED " .. tostring(GH.inprogress))
	-- if milling was performed, then pigments have been looted
	GH:doMillingDaily()
end

function GH:doMillingDaily()
	--[[Trade skill windows was opened. Check if own or NPC]]--
	-- count errors
	GH.notradeskillcount = GH.notradeskillcount or 0
	--avoid several calls (from several events) at once
	if type(GH.inprogress)==nil then
		GH.inprogress = time()
	elseif GH.inprogress == time() then
		return
	end
	-- if type(GH.db.profile.scribesupport)==nil then GH.db.profile.scribesupport=true end
	--if type(GH.db.profile.scribesupport)==nil then print(":(") end
	GH.db.profile.scribesupport = GH.db.profile.scribesupport or true
	--local name = GetTradeSkillLine(); --XXX language dependent, where is the id
	--GH:print("name "..tostring(name))
	if GH.db.profile.scribesupport and IsNPCCrafting() then
		--print("GetNumTradeSkills() " .. GetNumTradeSkills())
		if GetNumTradeSkills() == 0 then
			--try again
			GH.notradeskillcount = GH.notradeskillcount+1
			GH.inprogress =0 -- reset, that the next call will not be blocked
			if GH.notradeskillcount <= 10 then
				C_Timer.After(0.5, events.TRADE_SKILL_SHOW)
			else
				GH.notradeskillcount = 0
			end
		else
			GH.notradeskillcount = 0
			GH:destroy("scribe")
			for i=1,GetNumTradeSkills() do
				local sLink = GetTradeSkillItemLink(i)
				if sLink then
					local sID  = GH:getIDFromLink(sLink)
					if sID and sID == ids["Partial Receipt"] and not GetTradeSkillCooldown(i) then
						local skillName, skillType, numAvailable = GetTradeSkillInfo(i)
						-- print(skillName .. "  "  ..  skillType .. "  "  ..  numAvailable)
						--print(GH:getIDFromLink(sLink))
						--if numAvailable < 1 then
						-- unfortunately we cannot take numAvailable. this value will not be updated
						if GetItemCount(ids["Cerulean Pigment"], true)<4 then
							if GH:hasProfessionInscription() then
								MillingCommand()
							else
								GH:print("Ask somebody to mill herbs for you.")
							end
						else
							--DoTradeSkill(i)
							GH:setActionButton(sID,"/run DoTradeSkill(" .. i .. ")", "to create " .. skillName,nil,nil,nil,"scribe")
						end
					end
				end
			end
		end
	end
end

function GH:hasProfessionInscription()
	local res = false
	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions("player");
	local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(prof1)
	if skillLine == 773 then
		res = true
	else
		name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(prof2)
		if skillLine == 773 then
			res = true
		end
	end
	return res
end


--[[
used for milling macro:

#showtooltip Milling
/run MillingMacro()
/click GH_Button
]]
function MillingMacro()
	MillingCommand(true)
end
--[[
#showtooltip Milling
/mill
/click GH_Button
]]
--SLASH_MILLING1 = "/mill" SlashCmdList["MILLING"]=function(m)MillingMacro()end


function MillingCommand(hideButton)
	local herb = "";
	local mp = GH:MillingPriority()
	if #mp>0 then millingPriority=mp end
	--find herb in bag
	for j=1,#millingPriority do
		if GetItemCount(millingPriority[j], false)>=5 then
			herb = millingPriority[j]
			-- we have found at least 5 of one herb, but we have to make sure it's a stack of 5	
			
			local b = false
			for bag=0,4 do
				for slot=1,GetContainerNumSlots(bag) do
					local itemID = GetContainerItemID(bag, slot)
					if itemID then
						local _, count = GetContainerItemInfo(bag, slot)
						local itemName, itemLink, _, _, _, _, itemType = GetItemInfo(itemID)
						if itemName == herb then
							-- we have found the first stack of this herb, the macro will always mill the first stack
							if count >= 5 then
								-- everything is fine
							else
								print("|r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF There are "..GetItemCount(millingPriority[j], false).." x "..herb..", but the first stack is too small to mill!");
								herb = ""
							end
							b =true
						end
					end
					if b then break end
				end
				if b then break end
			end
			if #herb>0 then break end
		end
	end
	
	if hideButton then print("Milling "..herb) end
	if #herb > 0 then
		--print("|r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF Press button to mill " .. herb);
		GH:setActionButton(ids["Milling"],"/cast Milling\n/use " .. herb,"to mill " .. herb, true, ids["Milling Icon"],hideButton,"scribe")
	else
		print("|r|cFFFF7700<GarrisonHelper>|r|cFFFFFFFF Not enough resources and no herbs to mill found.");
	end
end

function GH:MillingPriority()
	--load saved herb list
	local res = {}
	for i=1,10 do
		if GH.db.profile["herb" .. i] then
			res[#res+1]=GetItemInfo(tonumber(GH.db.profile["herb" .. i]))
		end
	end
	return res
end

function GH_scribeWelcome()
	GH.db.profile.scribesupport = GH.db.profile.scribesupport or true
	if GH.db.profile.scribesupport and GH:hasBuilding(inscriptionID,2,true) then
		local _,duration = GetSpellCooldown(176513)
		if duration==0 then
			GH:print("Inscription daily not completed today");
		end
	end
end

-- register event listener
GarrisonHelper_Events:SetScript("OnEvent", function(self, event, ...) events[event](self, event, ...); end);
for event, func in pairs(events) do GarrisonHelper_Events:RegisterEvent(event); end


--[[
	Needed, because if cache is empty, this information have to be retrieved from server.
]]
function GH:PreLoadInscriptionItemInfos()
	for i=1,#millingPriority do
		GetItemInfo(millingPriority[i])
	end
end
GH:PreLoadInscriptionItemInfos() -- TODO move this to an event like addon loaded